function* generatePiDigits() {
  let q = 1n, r = 180n, t = 60n, i = 2n;
  for (;;) {
    const digit = ((i * 27n - 12n) * q + r * 5n) / (t * 5n);
    yield Number(digit);
    const u = i * 3n * (i++ * 3n + 1n) * 3n;
    r = u * 10n * (q * (i * 5n - 2n) + r - t * digit);
    q *= 10n * i * (i++ * 2n - 1n);
    t *= u;
  }
}

class PiChart {
  constructor() {
    this.digits = Array(10).fill(1); // Start with 1 to avoid division by zero
    this.total = 10;
    this.piGen = generatePiDigits();
    this.isRunning = false;
    this.chart = document.querySelector('.pie-chart');
    this.startBtn = document.getElementById('startBtn');
    this.pauseBtn = document.getElementById('pauseBtn');
    this.currentDigitEl = document.querySelector('.current-digit');
    this.digitsCountEl = document.querySelector('.digits-count');
    
    this.startBtn.addEventListener('click', () => this.start());
    this.pauseBtn.addEventListener('click', () => this.pause());
  }

  updateChart() {
    const percents = [];
    let accumulated = 0;
    
    this.digits.forEach((count, i) => {
      const percent = (count / this.total * 100).toFixed(2);
      accumulated += parseFloat(percent);
      percents.push(`var(--digit-${i}) ${accumulated - percent}% ${accumulated}%`);
    });

    this.chart.style.setProperty('--percent-0', `${this.digits[0]/this.total*100}%`);
    for(let i=0; i<10; i++) {
      const percent = (this.digits[i]/this.total*100).toFixed(2);
      this.chart.style.setProperty(`--percent-${i}`, `${percent}%`);
      this.chart.style.setProperty(`--digit-${i}`, `hsl(${i*36}, 80%, 60%)`);
    }
  }

  async start() {
    if (this.isRunning) return;
    this.isRunning = true;
    
    while (this.isRunning) {
      const { value: digit } = this.piGen.next();
      this.digits[digit]++;
      this.total++;
      
      this.currentDigitEl.textContent = digit;
      this.digitsCountEl.textContent = this.total - 10; // Subtract initial 1s
      this.updateChart();
      
      await new Promise(r => setTimeout(r, 50));
    }
  }

  pause() {
    this.isRunning = false;
  }
}

// Initialize when DOM loads
document.addEventListener('DOMContentLoaded', () => {
  const piChart = new PiChart();
  piChart.updateChart(); // Initial chart render
});